# -*- coding: utf8 -*-
# Copyright (c) 2021 Nicholas de Jong

__title__ = "elasticsearch_kibana_cli"
__author__ = 'Nicholas de Jong <contact@nicholasdejong.com>'
__version__ = '0.3.7'
__license__ = 'BSD2'

__cli_name__ = 'eskbcli'
__env_config_filename__ = 'ESKBCLI_CONFIG_FILENAME'
__search_split_count_default__ = 5
__search_split_bucket_limit__ = 10000
__search_default_timeout_seconds__ = 60




from signal import signal, SIGINT

from elasticsearch_kibana_cli import __cli_name__ as CLI_NAME
from elasticsearch_kibana_cli import __version__ as VERSION
from elasticsearch_kibana_cli.cli import click_cli
from elasticsearch_kibana_cli.exceptions.ElasticsearchKibanaCLIException import ElasticsearchKibanaCLIException


def sigint_handler(signal_received, frame):
    print('SIGINT received, exiting.')
    exit(1)


def eskbcli():
    signal(SIGINT, sigint_handler)

    try:
        click_cli.eskbcli_interface()
    except ElasticsearchKibanaCLIException as e:
        print('')
        print('{} v{}'.format(CLI_NAME, VERSION))
        print('ERROR: ', end='')
        for err in iter(e.args):
            print(err)
        print('')
        exit(9)




from elasticsearch_kibana_cli import __title__ as NAME
from elasticsearch_kibana_cli import __version__ as VERSION
from elasticsearch_kibana_cli import __cli_name__ as CLI_NAME
from elasticsearch_kibana_cli import __search_split_count_default__ as SEARCH_SPLIT_COUNT_DEFAULT

from elasticsearch_kibana_cli.utils.logger import Logger
from elasticsearch_kibana_cli.utils.config import ElasticsearchKibanaCLIConfig
from elasticsearch_kibana_cli.utils.connection import ElasticsearchKibanaCLIConnection
from elasticsearch_kibana_cli.utils.search import ElasticsearchKibanaCLISearch
from elasticsearch_kibana_cli.exceptions.ElasticsearchKibanaCLIException import ElasticsearchKibanaCLIException


logger = Logger(name=NAME).logging


class ElasticsearchKibanaInterface:

    config = None
    search = None
    connection = None

    def __init__(self, config_filename):
        logger.info('{} v{}'.format(CLI_NAME, VERSION))

        self.config = ElasticsearchKibanaCLIConfig(config_filename=config_filename).config
        logger.info('Loaded configuration filename {}'.format(config_filename))

    def list_searches(self):
        data = []
        if 'search_definitions' not in self.config:
            raise ElasticsearchKibanaCLIException('Configuration does not provide a "search_definitions" section.')
        else:
            for definition_key in self.config['search_definitions'].keys():
                data.append(definition_key)
            return data

    def show_search(self, name):
        if name not in self.list_searches():
            raise ElasticsearchKibanaCLIException('Configuration does not provide "search_definitions" '
                                                  'section with name "{}"'.format(name))
        else:
            return self.config['search_definitions'][name]

    def perform_search(self, name, split_count=None, ping_connection=True):

        search_config = self.show_search(name)

        if split_count is not None:
            search_config['splits'] = int(split_count)
        if 'splits' not in search_config.keys():
            search_config['splits'] = SEARCH_SPLIT_COUNT_DEFAULT

        self.kibana_connect(ping_connection=ping_connection)
        return self.search.msearch(**search_config)

    def kibana_connect(self, ping_connection=True, kbn_version=None):

        base_uri = 'http://127.0.0.1:9200'
        if 'base_uri' in self.config.keys():
            base_uri = self.config['base_uri']

        proxy_config = None
        if 'internal_proxy' in self.config.keys():
            proxy_config = self.config['internal_proxy']
            logger.debug('Using internal_proxy to connect {}'.format(base_uri))
        else:
            logger.debug('Using direct connection {}'.format(base_uri))

        self.connection = ElasticsearchKibanaCLIConnection(proxy_config=proxy_config)
        self.connection.attach(base_uri=base_uri, kbn_version=kbn_version)
        logger.debug('Connection definition setup {}'.format(self.connection.client_connect_address))

        if ping_connection is True:
            if self.connection.ping():
                logger.info('Ping okay {}'.format(self.connection.client_connect_address))
            else:
                raise ElasticsearchKibanaCLIException(
                    'Unable to ping Kibana endpoint', format(self.connection.client_connect_address)
                )
        else:
            logger.debug('Skipping ping endpoint check via {}'.format(self.connection.client_connect_address))

        self.search = ElasticsearchKibanaCLISearch(connection=self.connection)




import os
import json
import time
import hashlib
import tempfile
import requests
import threading
from bs4 import BeautifulSoup

from elasticsearch_kibana_cli import __title__ as NAME
from elasticsearch_kibana_cli import __cli_name__ as CLI_NAME
from elasticsearch_kibana_cli import __version__ as VERSION
from elasticsearch_kibana_cli.utils.logger import Logger
from elasticsearch_kibana_cli.utils.internal_proxy import ElasticsearchKibanaCLIInternalProxy
from elasticsearch_kibana_cli.exceptions.ElasticsearchKibanaCLIException import ElasticsearchKibanaCLIException


logger = Logger(name=NAME).logging


class ElasticsearchKibanaCLIConnection:

    user_agent = '{}/{}'.format(NAME, VERSION)
    kbn_version = None
    internal_proxy = None
    client_connect_address = None

    def __init__(self, proxy_config=None):
        if proxy_config is not None:
            self.internal_proxy = ElasticsearchKibanaCLIInternalProxy(config=proxy_config)

    def attach(self, base_uri, kbn_version=None):
        if self.internal_proxy:
            thread = threading.Thread(target=self.internal_proxy.start, args=[base_uri])
            thread.daemon = True
            thread.start()
            time.sleep(1)  # wait for internal_proxy thread to start <-- got to be a better way than this?
            self.client_connect_address = self.internal_proxy.client_connect_address
        else:
            self.client_connect_address = base_uri

        if kbn_version:
            self.kbn_version = kbn_version
        else:
            self.kbn_version = self.__kbn_version()

        return self.client_connect_address

    def ping(self, path='/app/monitoring'):
        url = '{}{}'.format(self.client_connect_address, path)
        headers = {'user-agent': self.user_agent}
        r = requests.get(url, headers=headers)
        if r.status_code == 200:
            return True
        return False

    def __kbn_version(self):
        metadata = self.__kbn_metadata()
        if 'version' not in metadata.keys():
            raise ElasticsearchKibanaCLIException('Unable to locate required version value in metadata')
        return metadata['version']

    def __kbn_metadata(self, use_cache=True):
        if self.client_connect_address is None:
            raise ElasticsearchKibanaCLIException(
                'Attempt to call __kbn_metadata before client_connect_address is set!'
            )

        cache_filename = '{}-metadata.cache'.format(self.__kbn_cache_basename())

        if use_cache and os.path.isfile(cache_filename):
            with open(cache_filename, 'r') as f:
                logger.debug('kbn_metadata read from cache file {}'.format(cache_filename))
                return json.load(f)

        headers = {'user-agent': self.user_agent}

        r = requests.get(self.client_connect_address, headers=headers)
        if r.status_code != 200:
            raise ElasticsearchKibanaCLIException('Unable to obtain data from {}'.format(self.client_connect_address))

        if 'defaultRoute' not in r.text:
            logger.debug('Attempting to collect kbn_metadata via "kbn-injected-metadata" element')
            soup = BeautifulSoup(r.content, 'html.parser')
            metadata_find = soup.find('kbn-injected-metadata')
        else:
            logger.debug('Attempting to collect kbn_metadata via "kbn-initial-state" element')
            url = '{}{}'.format(self.client_connect_address, '/app/kibana')
            r = requests.get(url, headers=headers)
            if r.status_code != 200:
                raise ElasticsearchKibanaCLIException('Unable to obtain data from {}'.format(url))
            soup = BeautifulSoup(r.content, 'html.parser')
            metadata_find = soup.find('kbn-initial-state')

        if not metadata_find:
            raise ElasticsearchKibanaCLIException('Unable to locate element for kbn_metadata '
                                                  'within {}'.format(self.client_connect_address))

        metadata = json.loads(metadata_find['data'])
        if use_cache:
            with open(cache_filename, 'w') as f:
                logger.debug('kbn_metadata write to cache file {}'.format(cache_filename))
                json.dump(metadata, f)
        return metadata

    def __kbn_cache_basename(self, base_path=None):
        if base_path is None:
            base_path = tempfile.gettempdir()
        if not os.path.exists(base_path):
            raise ElasticsearchKibanaCLIException('Cache base_path does not exist', base_path)

        return os.path.join(
            base_path,
            '{}-{}'.format(
                CLI_NAME,
                hashlib.md5(self.client_connect_address.encode('utf-8')).hexdigest()[0:8]
            )
        )
